@@ -1,4 +1,5 @@
 from hurry.resource.core import (Library,
+                                 libraries,
                                  ResourceInclusion,
                                  GroupInclusion,
                                  NeededInclusions)
@@ -11,8 +12,5 @@ from hurry.resource.core import (sort_inclusions_topological,
                                  sort_inclusions_by_extension,
                                  generate_code)
 
-try:
-    #BBB
-    from hurry.resource.wsgi import Middleware
-except ImportError:
-    pass
+from hurry.resource.core import (register_plugin,
+                                 get_current_needed_inclusions)
