@@ -1,9 +1,16 @@
-import os
+import os, sys
 from types import TupleType
+import pkg_resources
 
-from zope.interface import implements
-from zope import component
-
+try:
+    from zope.interface import implements
+    ZCA = True
+except ImportError:
+    # fallback in case zope.interface isn't present
+    def implements(iface):
+        pass
+    ZCA = False
+    
 from hurry.resource import interfaces
 
 EXTENSIONS = ['.css', '.kss', '.js']
@@ -11,10 +18,24 @@ EXTENSIONS = ['.css', '.kss', '.js']
 class Library(object):
     implements(interfaces.ILibrary)
     
-    def __init__(self, name):
+    def __init__(self, name, rootpath):
         self.name = name
+        self.rootpath = rootpath
+        self.path = os.path.join(caller_dir(), rootpath)
 
-class ResourceInclusion(object):
+# total hack to be able to get the dir the resources will be in
+def caller_dir():
+    return os.path.dirname(sys._getframe(2).f_globals['__file__'])
+
+def libraries():
+    for entry_point in pkg_resources.iter_entry_points(
+        'hurry.resource.libraries'):
+        yield entry_point.load()
+
+class InclusionBase(object):
+    implements(interfaces.IInclusion)
+
+class ResourceInclusion(InclusionBase):
     """Resource inclusion
     
     A resource inclusion specifies how to include a single resource in
@@ -109,8 +130,7 @@ class ResourceInclusion(object):
         return self.library.name, self.relpath
 
     def need(self):
-        needed = component.getUtility(
-            interfaces.ICurrentNeededInclusions)()
+        needed = _plugin.get_current_needed_inclusions()
         needed.need(self)
 
     def inclusions(self):
@@ -122,19 +142,16 @@ class ResourceInclusion(object):
         result.append(self)
         return result
 
-class GroupInclusion(object):
+class GroupInclusion(InclusionBase):
     """An inclusion used to group resources together.
 
     It doesn't define a resource itself.
     """
-    implements(interfaces.IInclusion)
-
     def __init__(self, depends):
         self.depends = depends
 
     def need(self):
-        needed = component.getUtility(
-            interfaces.ICurrentNeededInclusions)()
+        needed = _plugin.get_current_needed_inclusions()
         needed.need(self)
 
     def inclusions(self):
@@ -150,7 +167,9 @@ def normalize_inclusions(library, inclusions):
             for inclusion in inclusions]
 
 def normalize_inclusion(library, inclusion):
-    if interfaces.IInclusion.providedBy(inclusion):
+    if ZCA and interfaces.IInclusion.providedBy(inclusion):
+        return inclusion
+    if isinstance(inclusion, InclusionBase):
         return inclusion
     assert isinstance(inclusion, basestring)
     return ResourceInclusion(library, inclusion)
@@ -243,43 +262,63 @@ class NeededInclusions(object):
             html = html.replace('</body>', '%s</body>' % bottom, 1)
         return html
 
+
+class PluginNotImplemented(object):
+    """Plug-in point for hurry.resource.
+
+    Frameworks that want to plug into hurry.resource need to
+    implement these two methods.
+
+    There's no need to subclass this in your own code; just implement
+    the methods.
+    """
+    def get_current_needed_inclusions(self):
+        raise NotImplementedError(
+            "need to implement plugin.get_current_needed_inclusions()")
+
+    def get_library_url(self, library):
+        raise NotImplementedError(
+            "need to implement plugin.get_library_url()")
+
+_plugin = PluginNotImplemented()
+
+def register_plugin(plugin):
+    global _plugin
+    _plugin = plugin
+
+def get_current_needed_inclusions():
+    return _plugin.get_current_needed_inclusions()
+
 def mode(mode):
     """Set the mode for the currently needed resources.
     """
-    needed = component.getUtility(
-            interfaces.ICurrentNeededInclusions)()
+    needed = _plugin.get_current_needed_inclusions()
     needed.mode(mode)
 
 def bottom(force=False, disable=False):
     """Try to include resources at the bottom of the page, not just on top.
     """
-    needed = component.getUtility(
-            interfaces.ICurrentNeededInclusions)()
+    needed = _plugin.get_current_needed_inclusions()
     needed.bottom(force, disable)
 
 def rollup(disable=False):
-    needed = component.getUtility(
-        interfaces.ICurrentNeededInclusions)()
+    needed = _plugin.get_current_needed_inclusions()
     needed.rollup(disable)
 
 def render():
-    needed = component.getUtility(
-        interfaces.ICurrentNeededInclusions)()
+    needed = _plugin.get_current_needed_inclusions()
     return needed.render()
 
 def render_into_html(html):
-    needed = component.getUtility(
-        interfaces.ICurrentNeededInclusions)()
+    needed = _plugin.get_current_needed_inclusions()
     return needed.render_into_html(html)
 
 def render_topbottom():
-    needed = component.getUtility(
-        interfaces.ICurrentNeededInclusions)()
+    needed = _plugin.get_current_needed_inclusions()
     return needed.render_topbottom()
 
 def render_topbottom_into_html(html):
-    needed = component.getUtility(
-        interfaces.ICurrentNeededInclusions)()
+    needed = _plugin.get_current_needed_inclusions()
     return needed.render_topbottom_into_html(html)
 
 def apply_mode(inclusions, mode):
@@ -394,7 +433,7 @@ def render_inclusions(inclusions, library_urls=None):
         library_url = library_urls.get(library.name)
         if library_url is None:
             # if we can't find it, recalculate it
-            library_url = interfaces.ILibraryUrl(library)
+            library_url = _plugin.get_library_url(library)
             if not library_url.endswith('/'):
                 library_url += '/'
             library_urls[library.name] = library_url
@@ -430,7 +469,8 @@ def generate_code(**kw):
     result.append("")
     # define libraries
     for library in libraries:
-        result.append("%s = Library('%s')" % (library.name, library.name))
+        result.append("%s = Library('%s', '%s')" %
+                      (library.name, library.name, library.rootpath))
     result.append("")
 
     # sort inclusions in the order we want them to be
